"""
For more details, see readme.md
"""
import os
import csv
import pathlib

parent = pathlib.Path(__file__).parent.resolve()

csv_list = [d for d in os.listdir(parent / "../csv_downloaded/") if not d.startswith("readme")]
csv_list.sort()

# open out csv
print("Merging datasets")
with open(parent / "polymer_membrane_entities_all_annotations.csv", "w", encoding="utf-8") as outfile:
    
    writer = csv.writer(outfile)
    # variable for cross check
    n_lines_read = 0

    for idx, incsv in enumerate(csv_list):

        with open(parent / "../csv_downloaded" / incsv, "r", encoding="utf-8") as infile:
            reader = csv.reader(infile)
            dataset = list(reader)
            # checks
            assert dataset[0][0] == "Identifier"
            assert dataset[1][0] == "Entity ID"

            if idx == 0:
                # use header, remove first line w/out useful info
                n_lines_read += len(dataset) - 1
                writer.writerows(dataset[1:])
            else:
                n_lines_read += len(dataset) - 2
                writer.writerows(dataset[2:])

# filtering (separated in order to do a check and counting)
membrane_header = [
    "PDBTM_annotation_identifier",
    "MemProtMD_annotation_identifier",
    "mpstruct_annotation_identifier",
    "mpstruct_annotation_lineage_name",
    "OPM_annotation_identifier",
    "OPM_annotation_lineage_name",
]
print("Refactoring columns")
with open(parent / "polymer_membrane_entities_all_annotations.csv", "r", encoding="utf-8") as newfile:
    reader = csv.reader(newfile)
    dataset = list(reader)
    # independent check
    assert len(dataset) == n_lines_read

    # remove those annotation not related to membrane and create
    with open(parent / "polymer_membrane_entities.csv", "w", encoding="utf-8") as outfile, open(parent / "excluded_lines.csv", "w", encoding="utf-8") as file_excluded_lines:
        writer = csv.writer(outfile)
        # new header
        new_row = dataset[0][:-4] + membrane_header # list concatenation to add last columns; -4 and not -3 because the header finishes with a ","
        n_columns = len(new_row)
        print(f"New columns (n={n_columns}): {new_row}\n")
        for idx, row in enumerate(dataset[1:]):
            if len(row) < 12:
                # handle errors in the CSV
                s = f"WARNING: line skipped caused less than 12 entries -> {idx:d}: {row}\n"
                file_excluded_lines.write(s)
                continue
            if row[0]:
                # if first is different from the empty string, then it is related to a polymer
                assert len(new_row) == n_columns, print(idx, new_row)
                writer.writerow(new_row)
                new_row = row[:-3] + ['', '', '', '', '', '']
            if row[11] == "PDBTM":
                # saving "annotation Identifier"
                new_row[-6] = f"{row[9]}"
            if row[11] == "MemProtMD":
                # saving "annotation Identifier"
                new_row[-5] = f"{row[9]}"
            if row[11] == "mpstruc":
                new_row[-4] = f"{row[9]}"
                new_row[-3] = f"{row[10]}"
            if row[11] == "OPM":
                new_row[-2] = f"{row[9]}"
                new_row[-1] = f"{row[10]}"
        writer.writerow(new_row)

# Another check to ensure the number of polymer entities is equal to the starting one
print("Last check")
with open(parent / "polymer_membrane_entities.csv", "r", encoding="utf-8") as lastfile:
    reader = csv.reader(lastfile)
    assert len(list(reader)) == 28151 + 1, print(len(list(reader))) # result given by RCSB search
