"""
Function to Parse CATH files
"""
def parse_domain(path_file):
    """Reads cath-domain-list.txt"""
    output_dict = {}
    with open(path_file, "r", encoding="utf-8") as file:
        for line in file.readlines():
            if line.startswith("#"):
                continue
            # split the row into parts
            line_split = line.split(" ")
            row = [el.strip("\n") for el in line_split if el]
            assert len(row) == 12, print(row) # see README for cath-domain-list
            output_dict[row[0]] = row[1:]
    return output_dict

def parse_boundary(path_file):
    """
    Read the cath-domain-boundaries-seqreschopping file
    and save the result into a dictionarys structure.
    The disctionary has as keys the PDBID of each domain,
    then the relative value is a nested dictionary with keys
    the CATH code for each domain ("10gsA01") and value the 
    sequence portions ("2-78,187-208", relative to the same chain).

    Chain ID is the one used in the CATH, which is auth_asym_id

    Parameters
    ----------
    path_file : str or pathlib
        Path to cath-domain-boundaries-seqreschopping file

    Returns
    -------
    Dict
        See function description
    """
    pdb_dict = {}
    with open(path_file, "r", encoding="utf-8") as file:
        first_line = file.readline()
        sline = first_line.split("\t") # separated by tab
        pdb_id_temp = sline[0][:4]
        pdb_dict[pdb_id_temp] = {sline[0]:sline[1].strip("\n")}

        for line in file.readlines():
            sline = line.split("\t")
            pdb_id = sline[0][:4]

            if pdb_id != pdb_id_temp:
                # if a new PDB is found
                pdb_dict[pdb_id] = {}
                pdb_id_temp = pdb_id

            sline[1] = sline[1].strip("\n")
            pdb_dict[pdb_id][sline[0]] = sline[1]

    return pdb_dict
