"""
See README for more details
"""
import csv
import pathlib

from CATHParser import parse_domain, parse_boundary

parent = pathlib.Path(__file__).parent.resolve()

# read csv
print("Load CSV")
with open(parent / "../0_csv_merge/polymer_membrane_entities.csv", "r", encoding="utf-8") as csvfile:
    reader = csv.reader(csvfile)
    csvdatabase = list(reader)

# read domains classification and domain sequences
print("Load CATH files")
domain_dict = parse_domain(parent / "cath_data/cath-domain-list-v4_3_0.txt")
seqres_dict = parse_boundary(parent / "cath_data/cath-domain-boundaries-seqreschopping-v4_3_0.txt")

print("Write new CSV")
with open(parent / "polymer_membrane_entities_cath_domains.csv", "w", encoding="utf-8") as outfile, open(parent / "polymer_membrane_entities_not_in_cath.txt", "w", encoding="utf-8") as not_cath_file, open(parent / "polymer_membrane_entities_not_cath_code.txt", "w", encoding="utf-8") as no_cath_code_file:

    not_cath_file.write("# PDBs without CATH entry\n")
    no_cath_code_file.write("# CATH entry with domain division but no CATH classification\n")

    writer = csv.writer(outfile)
    # new header
    cath_header = [
        "CATH_domain_number",
        "CATH_domain_codes",
        "CATH_domain_seqreschopping_json"
    ]
    new_row = csvdatabase[0] + cath_header
    writer.writerow(new_row)

    obs_counter = 0
    domains_counter = 0
    for entry in csvdatabase[1:]:
        assert entry[0] != "Entity ID"
        new_row = entry

        # search in seqres
        pdb_id = entry[0][:4].lower()
        try:
            boundaries_before_filtering = seqres_dict[pdb_id] # dictionary
        except KeyError:
            not_cath_file.write(f"{pdb_id}\n")
            continue

        # store auth_asym_id, because only the chain of the membrane protein
        # is sampled to get info about domain subdivision
        auth_asym_id = entry[7]

        bounderies_2insert = {}
        cath_domain_code_2insert = {}
        for bd, value in boundaries_before_filtering.items():
            # here we iterate over CATH codes

            # filtering non-membrane chains
            if bd[4] != auth_asym_id:
                continue
            # filtering OBSOLETE PDBs
            if value[-1] == "1000.000":
                # -1 == resolutions column in CATH-domain
                obs_counter += 1
                continue
            try:
                # save up until 6th column from the cath-domain file
                # see README-cath-list
                cath_domain_code_2insert[bd] = domain_dict[bd][:5]
            except KeyError:
                no_cath_code_file.write(f"{bd}\n")
                continue

            bounderies_2insert[bd] = value

        len_domains = len(cath_domain_code_2insert)
        len_boundaries = len(bounderies_2insert)
        assert len_boundaries == len_domains
        new_row += [f'{len_domains:d}', f'{cath_domain_code_2insert}', f'{bounderies_2insert}']

        domains_counter += len_domains

        if len_domains == 0:
            continue

        writer.writerow(new_row)

print(f"Obsolete PDBs filtered: {obs_counter}")
print(f"Number of domains: {domains_counter}")
