"""
For details see README.md
"""
import pathlib
import json
import csv

from CATHtree import CATHTree, CATHDomain

parent = pathlib.Path(__file__).parent.resolve()

with open(parent / "../1_cath_merge/polymer_membrane_entities_cath_domains.csv", "r", encoding="utf-8") as csvfile:
    csvreader = csv.reader(csvfile)
    database = list(csvreader)

# initialize object to fill
root = CATHTree(cath_code="0")

# read dataset and fill the tree
for row in database:
    if row[0] == "Entity ID":
        continue

    # read json containing all domains and CATH codes
    # json requires " instead of ' to highlight keys
    domains = json.loads(row[16].replace("'", "\""))
    if "NMR" in row[1]:
        resolution = "999.000"
    elif row[1] in ('ELECTRON CRYSTALLOGRAPHY', 'ELECTRON MICROSCOPY'):
        # Some electro microscopy experiments provide two
        # resolutions, take only the first one
        resolution = row[2].split(",")[0]
    else:
        resolution = row[2]

    for domain_name, cath_class in domains.items():
        # C level
        idx_branch_C = root.get_branch_index_from_code(cath_class[0])
        if idx_branch_C is None:
            root.add_branch(CATHTree(cath_code=cath_class[0]))
            idx_branch_C = root.get_branch_index_from_code(cath_class[0])
        node_C = root.get_branches()[idx_branch_C]
        # A level
        idx_branch_A = node_C.get_branch_index_from_code(cath_class[1])
        if idx_branch_A is None:
            node_C.add_branch(CATHTree(cath_code=cath_class[1]))
            idx_branch_A = node_C.get_branch_index_from_code(cath_class[1])
        node_A = node_C.get_branches()[idx_branch_A]
        # T level
        idx_branch_T = node_A.get_branch_index_from_code(cath_class[2])
        if idx_branch_T is None:
            node_A.add_branch(CATHTree(cath_code=cath_class[2]))
            idx_branch_T = node_A.get_branch_index_from_code(cath_class[2])
        node_T = node_A.get_branches()[idx_branch_T]
        # H level
        idx_branch_H = node_T.get_branch_index_from_code(cath_class[3])
        if idx_branch_H is None:
            node_T.add_branch(CATHTree(cath_code=cath_class[3]))
            idx_branch_H = node_T.get_branch_index_from_code(cath_class[3])
        node_H = node_T.get_branches()[idx_branch_H]

        if not node_H.is_cath_code_complete():
            raise RuntimeError(f"The following node have not a complete CATH code: {node_H.get_cath()}")

        # Logic to insert a new structure
        hyperfamily_group_domains = node_H.get_domains()
        if len(hyperfamily_group_domains) == 0:
            # add entry if no domains are found
            node_H.add_domain(CATHDomain(domain_name=domain_name, cath_classification=cath_class, resolution=resolution))
        else:
            # control if homology cluster is already present
            domain_not_present = True
            for idx, domain in enumerate(hyperfamily_group_domains):
                if cath_class[4] == domain.get_homology35_id():
                    if float(resolution) < float(domain.get_resolution()):
                        node_H.del_domain(idx)
                        node_H.add_domain(CATHDomain(domain_name=domain_name, cath_classification=cath_class, resolution=resolution))
                    domain_not_present = False
                    break
            if domain_not_present:
                node_H.add_domain(CATHDomain(domain_name=domain_name, cath_classification=cath_class, resolution=resolution))


# save results and check that there are no
# domains with same CATH code and same clustering ID
with open(parent / "domains_35_homology_filtered.csv", "w", encoding="utf-8") as outfile:
    n_domains = 0
    writer = csv.writer(outfile)
    for C_code in root.get_branches():
        for A_code in C_code.get_branches():
            for T_code in A_code.get_branches():
                for H_code in T_code.get_branches():
                    domains = H_code.get_domains()
                    n_domains += len(domains)
                    # check if there are duplicate homologies
                    homologies = [d.get_homology35_id() for d in domains]
                    assert len(homologies) == len(set(homologies)), print(homologies)
                    for dom in domains:
                        writer.writerow([dom.get_domain_info()] + dom.get_domain_cath())

print("Domains: ", n_domains)

print("Writing CSV with all info")
with open(parent / "domains_35_homology_filtered.csv", "r", encoding="utf-8") as domains, open(parent / "polymer_membrane_entities_cath_35homology_filtered.csv", "w", encoding="utf-8") as outfile:

    reader = csv.reader(domains)
    domains_filtered = {}
    for row in reader:
        domains_filtered[row[0]] = row[1:]
    
    writer = csv.writer(outfile)
    writer.writerow(database[0])

    dom_couter = 0

    for row in database[1:]:

        domains_not_filtered_single_entry = json.loads(row[16].replace("'", "\""))
        seqres_not_filtered_single_entry = json.loads(row[17].replace("'", "\""))
        domain_2save = {}
        seqres_2save = {}
        for dom in domains_not_filtered_single_entry:
            try:
                domains_filtered[dom]
                present = True
            except KeyError:
                present = False
            
            if present:
                domain_2save[dom] = domains_not_filtered_single_entry[dom]
                seqres_2save[dom] = seqres_not_filtered_single_entry[dom]

        len_new_domains = len(domain_2save)
        len_new_seqres = len(seqres_2save)
        assert len_new_domains == len_new_seqres
        if len_new_seqres != 0:
            out_cath = [f'{len_new_seqres}', f'{domain_2save}', f'{seqres_2save}']
        else:
            out_cath = []

        if out_cath:
            # if the entry is non-zero
            dom_couter += int(out_cath[0])
            new_row = row[:-3] + out_cath
            assert len(new_row) == 18, print(new_row)
            writer.writerow(new_row)

print("CHECK Domains: ", dom_couter)
