"""
Plot for the distribution of domain len based on CATH info
"""
import csv
import json
import pathlib

import matplotlib
import matplotlib.pyplot as plt

from utils import (
    domain_segments_parser,
    refined_text,
    remove_right_upper_axes,
    set_ticks,
    fancy_grid,
)

parent = pathlib.Path(__file__).parent.resolve()

# load data
trans_mono_file = parent / "classification" / "transmembrane_monotopic_list.csv"
with open(trans_mono_file, "r") as data_file:
    reader = csv.reader(data_file)
    next(reader)
    mono = []
    trans = []
    for row in reader:
        domains = domain_segments_parser(json.loads(row[2].replace("'", '"')))
        for dom, segs in domains.items():
            dom_len = 0
            for piece in segs:
                dom_len += piece[1] - piece[0] + 1
            if row[1] == "mono":
                mono.append(dom_len)
            elif row[1] == "trans":
                trans.append(dom_len)
            else:
                raise RuntimeError("Unknown type")

# Plot
matplotlib.rcParams["font.size"] = 24
matplotlib.rcParams["text.usetex"] = True
matplotlib.rcParams["font.family"] = "modern computer"

fig, ax = plt.subplots(1, figsize=(10, 10))

edges = 50
# ax.hist(mono, bins=edges, alpha=0.4, color="#3498DB")
# ax.hist(trans, bins=edges, alpha=0.4, color="red")
ax.hist([mono, trans], bins=edges, alpha=0.4, stacked=True)

refined_text(
    ax,
    f"Mono ${len(mono):d}$\nTrans ${len(trans):d}$",
    x=0.8,
    y=0.85,
)

# refinements
remove_right_upper_axes(ax)
set_ticks(ax, {"x_major": 150, "x_minor": 25, "y_major": 25})
fancy_grid(ax, {"y_major": True})
ax.set_title("Domain length distributions (stacked)", pad=15)
fig.savefig(parent / "plots" / "length_mono_trans_distr.pdf", bbox_inches="tight")
