# Membrane domain database
> Last update: 2023-02-28 16:19:57
- [Membrane domain database](#membrane-domain-database)
  - [Header](#header)
  - [Pipeline](#pipeline)
    - [0 - Download tables from RCSB](#0---download-tables-from-rcsb)
  - [1 - Retrieve info from CATH](#1---retrieve-info-from-cath)
  - [2 - Homology filter](#2---homology-filter)


## Header
Description of the header for the database file `polymer_membrane_entities_cath_35homology_filtered.csv` (see below for more details on why this name and how it is generated). Each entry is presented as: `name (source, type): description`. The description collects info from the source of the data providing, when available, the link reference.

For completeness, sources are the following: [RCSB](https://www.rcsb.org/) and [CATH](https://www.cathdb.info/)

- `Entity ID` (RCSB, str): composed as `PDBID_ENTITYID`, where the `PDBID` is the PDB unique code and `ENTITYID` is the [entity level](https://www.rcsb.org/docs/general-help/identifiers-in-pdb#entity-level-identifiers-) regarding the chemical (biomolecule, non-polymer etc.)
- Experimental Method (RCSB, str): corresponding to `experimental_method` value inside `rcsb_entry_info`
- Resolution (RCSB, str): experiment resolution in angstrom. Not provided for NMR experiments and some Cryo-EM has two resolutions
- Entry Polymer Types (RCSB, str): `entity_poly` > `rcsb_entity_polymer_type`; possible values: "Nucleic acid (only)" "Oligosaccharide (only)" "Other" "Protein (only)" "Protein/NA" "Protein/Oligosaccharide"
- Structure Determination Methodology (RCSB, str): experimental or computational
- Structure Title (RCSB, str): `rcsb_primary_citation` > `title`
- Asym ID (RCSB, str): RCSB assigned chain ID
- Auth Asym ID (RCSB, str): Author assigned chain ID at the time of deposit
- Source Organism (RCSB, str): homonym entry 
- PDBTM_annotation_identifier (RCSB, str): `rcsb_polymer_entity_annotation` > `annotation_id` for PDBTM annotation entry
- MemProtMD_annotation_identifier (RCSB, str): `rcsb_polymer_entity_annotation` > `annotation_id` for MemProtMD annotation entry
- mpstruct_annotation_identifier (RCSB, str): `rcsb_polymer_entity_annotation` > `annotation_id` for mpstruct annotation entry
- mpstruct_annotation_lineage_name: (RCSB, str): `rcsb_polymer_entity_annotation` > `annotation_lineage` for mpstruct annotation entry
- OPM_annotation_identifier (RCSB, str): `rcsb_polymer_entity_annotation` > `annotation_id` for OPM annotation entry
- OPM_annotation_lineage_name (RCSB, str): `rcsb_polymer_entity_annotation` > `annotation_lineage` for OPM annotation entry
- CATH_domain_number (computed, int): number of domains contained in the polymer after filtering
- CATH_domain_codes (CATH, json): json with keys representing CATH domain names (e.g. `1ucsA01`, hence PDBID `1ucs`, chain Auth Asym ID `A` and domain number 1) having as values the CATH classification code (e.g. `1.20.148.1`)
- CATH_domain_seqreschopping_json (CATH, json): keys similar to `CATH_domain_codes` and having as values the sequence sections that represent that domain. Note that sequences can be non-contiguous too. The notation is the one used in the `cath-domain-boundaries-seqreschopping` file, where stretches of contiguous residues are separated by the symbol `-`, while jumps in the sequence (but still same domain) are separated by a `:` symbol

## Pipeline
### 0 - Download tables from RCSB
In the `csv_downloaded` there are stored a series of CSV files containing entries relative to memebrane protein. These CSV are obtained thrrought the powerful search graphical engine provided by RCSB. One can access this programmatically by means of REST API too.

The following conditions are used to perform the research:
- `Annotation Type` is PDBTM or MemProtMD or OPM or mpsrtuct. Substantially, this select all PDB entries which are classified as memebrane protein by the database.
- Return `Polymer Entities`. This is done because we are interested in the polymer entries rather than the whole protein.
- Only experimentally determined structures are used.
Using the REST API, one can use the following JSON:
```
{
  "query": {
    "type": "group",
    "nodes": [
      {
        "type": "terminal",
        "service": "text",
        "parameters": {
          "attribute": "rcsb_polymer_entity_annotation.type",
          "operator": "exact_match",
          "value": "PDBTM"
        }
      },
      {
        "type": "terminal",
        "service": "text",
        "parameters": {
          "attribute": "rcsb_polymer_entity_annotation.type",
          "operator": "exact_match",
          "value": "MemProtMD"
        }
      },
      {
        "type": "terminal",
        "service": "text",
        "parameters": {
          "attribute": "rcsb_polymer_entity_annotation.type",
          "operator": "exact_match",
          "value": "OPM"
        }
      },
      {
        "type": "terminal",
        "service": "text",
        "parameters": {
          "attribute": "rcsb_polymer_entity_annotation.type",
          "operator": "exact_match",
          "value": "mpstruc"
        }
      }
    ],
    "logical_operator": "or",
    "label": "text"
  },
  "return_type": "polymer_entity",
  "request_options": {
    "paginate": {
      "start": 0,
      "rows": 25
    },
    "results_content_type": [
      "experimental"
    ],
    "sort": [
      {
        "sort_by": "score",
        "direction": "desc"
      }
    ],
    "scoring_strategy": "combined"
  }
}
```

The returned information is customized into a tabular format. See the CSV file to understand which fields are selected and compare them with the description provided in [Header](#header).

In this phase the following is performed:
1. read and put together all files
2. remove all rows without information on membrane entry by looking at the `Annotation Identifier`. Only rows with PDBTM, MemProtMD, mpstruct or OPM are maintained.

Outputs:
- `polymer_membrane_entity_all_annotations.csv`: intermediate step with all csv merrged and all annotations
- `polymer_membrane_entity.csv`: final file with only membrane annotation and header refactored. Each row correspond to a polymer entry 
- `excluded_lines.csv`: annotations that are related to another database that has less than 12 columns. Not relevant, used just as a negative test.

Checks:
- check that final number of polymers is the same from the starting search
- visually check consistency from `_all_annotations.csv` to the final one


## 1 - Retrieve info from CATH
In this section we retrieve information about domain separation from the CATH for every classified chain. One thing to be under control is the notation used by the CATH, because [chain identifiers](https://www.rcsb.org/docs/general-help/identifiers-in-pdb#macromolecular-instance-id) used by the CATH are the `auth_asym_id`, hence those assigned by the author of a PDB file at the time of deposit.

Procedure:
1. Load `cath-domain-list-v4_3_0.txt` and `cath-domain-boundaries-seqreschopping-v4_3_0.txt` file download from the [CATH website](http://download.cathdb.info/cath/releases/all-releases/v4_3_0/) (`v4.3.0`). The [first file](http://download.cathdb.info/cath/releases/all-releases/v4_3_0/cath-classification-data/README-cath-list-file-format.txt) contains info regarding the CATH code for each domain entry. While [the second](http://download.cathdb.info/cath/releases/all-releases/v4_3_0/cath-classification-data/README-cath-domain-boundaries-file-format.txt) describe the starting residue and the final one for each domain in the chain. Note: data are loaded as dictionary, memory-expensive but fast to search into (see `CATHParser.py` file).
2. Add new columns regarding CATH
3. Filter OBSOLETE entries (`1000.000` value in the resolution entry in `cath-domain-list-v4_3_0.txt`)
4. recover info about domain and sequence section once a PDB is selected. These info are saved as dictionaries (or JSON) in the corresponding column (see [Header](#header))

Output:
- `polymer_membrane_entities_cath_domains.csv`: output of this section
- `polymer_membrane_entities_not_in_cath.txt`: PDB that are not yet classified by the CATH database
- `polymer_membrane_entities_not_cath_code.txt`: those domains which are identified (domain position in the sequence is known from the `cath-domain-boundaries-seqreschopping-v4_3_0.txt` file) but has no CATH code yet

Check:
- visual check for consistency

## 2 - Homology filter
This passage filter the domains, characterized by their CATH code, by homology. Hence, sequences that shares $\geq s\%$ of their sequences are considered homologous. This is used to avoid redundancy in the dataset and avoid similar structure related, possibly, to similar functions. 

Procedure:
- when two resolutions are provided (in Cryo-EM experiments) select the first one
- create a tree by running through the dataset and a tree (each branch is one of the code assigned by CATH). These structure allow for a fast search for those domains with a given CATH code.
- Filter by homology: here we use the clustering procedue provided by the CATH team. Column 6 in the `cath-domain-list-v4_3_0.txt` gives the `S35 sequence cluster number`. Basically, domains with the same CATH code are clustered by homology, if they share more than $35\%$ of the sequence then belongs to the same cluster.
- The homology filter select the domain in a cluster with better resolution. 

Output:
- `polymer_membrane_entities_cath_35homology_filtered.csv`: final dataset
- `domains_35_homology_filtered.csv`: list of domains after clustering filter

Checks:
- programmatic check that there are no domains with same CATH and same clustering number
- check on domains filtered and save afterwards in the final file
- check that number of columns are 18
